from abc import ABCMeta
from typing import Dict

from .Command import Command
from ..state import World


class Logic:

    def __init__(self, world: World):
        self.__commands = {}  # type: Dict[int, Command]
        self.__world = world

    @property
    def world(self) -> World:
        return self.__world

    def addCommand(self, command: Command):
        self.__commands[command.priority()] = command

    def executeCommands(self):
        commands = self.__commands.copy()
        self.__commands.clear()
        priorities = sorted(commands.keys())
        for priority in priorities:
            command = commands[priority]
            if not command.check(self):
                continue
            command.execute(self)

    def getSetLayerCellCommand(self, layer: str) -> ABCMeta:

        from .command.SetGroundCellCommand import SetGroundCellCommand
        from .command.SetImpassableCellCommand import SetImpassableCellCommand
        from .command.SetObjectsCellCommand import SetObjectsCellCommand
        from .command.SetUnitsCellCommand import SetUnitsCellCommand
        setLayerValueCommand = {
            "ground": SetGroundCellCommand,
            "impassable": SetImpassableCellCommand,
            "objects": SetObjectsCellCommand,
            "units": SetUnitsCellCommand,
        }
        return setLayerValueCommand[layer]

