import logging
from math import sqrt

from core.constants import CellValue, UnitClass
from core.state import World, GameState, Unit
from tools.vector import vectorMulI, vectorDivI, vectorSubI
from ui import UserInterface, Theme
from ui.mode import EditGameMode

logging.basicConfig(level=logging.INFO, format='\r%(asctime)s %(filename)s:%(lineno)d: %(message)s')


# Create a basic game state
world = World((50, 36))
worldCenter = vectorDivI(world.size, (2, 2))
ground = world.ground.cells
for y in range(0, 24):
    for x in range(0, 40):
        x0, y0 = x - 20, y - 12
        d = sqrt(x0 * x0 + 3 * y0 * y0)
        if d < 15:
            ground[x0 + worldCenter[0] - 1, y0 + worldCenter[1] - 1] = CellValue.GROUND_EARTH
x, y = worldCenter
for playerId in range(5):
    world.units.setUnit((x, y), CellValue.UNITS_UNIT, Unit(UnitClass.WORKER, playerId))
    x += 1
state = GameState(world)

# Create a user interface and run it
theme = Theme()
tileSize = theme.getTileset("ground").tileSize
theme.viewSize = vectorMulI((27, 15), tileSize)
userInterface = UserInterface(theme)
gameMode = EditGameMode(theme, state)
userInterface.setGameMode(gameMode)
worldSize = vectorMulI(world.size, tileSize)
view = vectorDivI(vectorSubI(worldSize, theme.viewSize), (2, 2))
gameMode.viewChanged(view)
userInterface.run()
userInterface.quit()
